/* 
    ntuxplayer

    Copyright (C) 2000 Marcus Metzler (mocm@convergence.de)
    for convergence integrated media

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include <getopt.h>
#include <curses.h>
#include "dvb_formats.h"

int yskip = 0;
int volume = 200;
int nopat = 0;


void set_volume(int fda, int vol)
{
	int ans;
	audioMixer_t am;
	static int novol = 0;

	if (novol) return;
	if (vol == volume) return;

	if (vol > 255) vol = 255;
	if (vol < 0  ) vol = 0;
	volume = vol;
	am.volume_left = vol;
	am.volume_right = vol;

	if ( (ans = ioctl(fda,AUDIO_SET_MIXER, &am) < 0)){
		novol = 1;
		return;
	}
}

void stop_curses(void)
{
    clear();
    refresh();
    endwin();
}

void curses_init(void)     /* for kbd input */
{
    initscr();
    refresh();
    cbreak();
    noecho();
    nonl();
    intrflush(stdscr, FALSE);
    keypad(stdscr, TRUE);
    nodelay(stdscr, TRUE);

    atexit(stop_curses);
}



void play_file_video(int filefd, int fd, int fda, int next)
{
	char buf[BUFFY];
	int count,c;
	int written;
	struct pollfd pfd[NFD];
	int stopped = 0;
	uint64_t length = 0;
	uint64_t l = 0;

	pfd[0].fd = STDIN_FILENO;
	pfd[0].events = POLLIN;
	
	pfd[1].fd = fd;
	pfd[1].events = POLLOUT;

	if (fda) audioSetAVSync(fda, true);
	if (!next){
		videoSelectSource(fd,VIDEO_SOURCE_MEMORY);
		videoPlay(fd);
	}		
	
	length = lseek(filefd, 0, SEEK_END);
	lseek(filefd,0,SEEK_SET);
	printf("File length: %2.2f MB\r\n",(double)length/1024/1024);
	yskip++;
	count = read(filefd,buf,BUFFY);
	write(fd,buf,count);
	if (count > 0)  l+= count;
	
	set_volume(fda, volume);
	mvprintw(yskip+1,30,
		 "Volume: %d",volume);
	mvprintw(yskip,30, "playback started  ");
	refresh();

	while ( l < length ){
		if ( (count = read(filefd,buf,BUFFY)) < 0){
			perror("AUDIO DEVICE: ");
			exit(1);
		}		
		
		if (count > 0) l+= count;
		mvprintw(yskip,0,"read %02.2f%%",100.0*l/length);
		refresh();

		written = 0;
		while(count >0 && written < count){
			if (poll(pfd,NFD,100)){
				if (pfd[1].revents & POLLOUT){
					c = write(fd,buf+written,
							count-written);
					if (c > 0) written += c;
				}
				if (pfd[0].revents & POLLIN){
					int cc = getch();
					switch(cc){
					case 'z':
						mvprintw(yskip,30,
							 "playback frozen   ");
						audioPause(fda);
						videoFreeze(fd);
						stopped = 1;
						break;
					case 's':
						mvprintw(yskip,30,
							 "playback stopped  ");
						videoStop(fd);
						audioStop(fda);
						stopped = 1;
						break;
			
					case 'c':
						mvprintw(yskip,30,
							"playback continued ");
						audioContinue(fda);
						videoContinue(fd);
						audioSetAVSync(fda, true);
						stopped = 0;
						break;

					case 'p':
						mvprintw(yskip,30,
							 "playback started  ");
						audioSetAVSync(fda, true);
						audioPlay(fda);
						videoPlay(fd);
						stopped = 0;
						break;

					case 'f':
						mvprintw(yskip,30,
							 "fast forward      ");
						audioSetAVSync(fda, false);
						audioPause(fda);
						videoFastForward(fd,0);
						stopped = 0;
						break;

					case 'm':
						mvprintw(yskip,30,
							 "slow motion       ");
						audioSetAVSync(fda, false);
						audioPause(fda);
						videoSlowMotion(fd,2);
						stopped = 0;
						break;
						
					case '+':
						set_volume(fda, volume+1);
						mvprintw(yskip+1,30,
							 "Volume: %d",volume);
						break;

					case '-':
						set_volume(fda, volume-1);
						mvprintw(yskip+1,30,
							 "Volume: %d",volume);
						break;


					case 'q':
						videoContinue(fd);
						exit(0);
						break;
					}
				}
				
			}
		}
	}
	if (fda) audioSetAVSync(fda, false);
}

void find_avpids(int fd, uint16_t *vpid, uint16_t *apid)
{
	uint8_t buf[IN_SIZE];
	int count;
	int i;	
	int off =0;

	while ( *apid == 0 || *vpid == 0){
		count = read(fd, buf, IN_SIZE);
		for (i = 0; i < count-7; i++){
			if (buf[i] == 0x47){
				if ((buf[i+1] & 0xF0) == 0x40){
					off = 0;
					if ( buf[3+i] & 0x20)//adapt field?
						off = buf[4+i] + 1;
					switch(buf[i+7+off]){
					case VIDEO_STREAM_S ... VIDEO_STREAM_E:
						*vpid = get_pid(buf+i+1);
						break;
					case AUDIO_STREAM_S ... AUDIO_STREAM_E:
						*apid = get_pid(buf+i+1);
						break;
					}
				}
				i += 187;
			}
			if (*apid != 0 && *vpid != 0) break;
		}
	}
}

void find_bavpids(uint8_t *buf, int count, uint16_t *vpid, uint16_t *apid)
{
	int i;	
	int founda = 0;
	int foundb = 0;
	int off = 0;

	for (i = 0; i < count-7; i++){
		if (buf[i] == 0x47){
			if ((buf[i+1] & 0xF0) == 0x40){
				off = 0;
				if ( buf[3+i] & 0x20)  // adaptation field?
					off = buf[4+i] + 1;
				
				if (buf[off+i+4] == 0x00 && 
				    buf[off+i+5] == 0x00 &&
				    buf[off+i+6] == 0x01){
					switch(buf[off+i+7]){
					case VIDEO_STREAM_S ... VIDEO_STREAM_E:
						*vpid = get_pid(buf+i+1);
						foundb=1;
						break;
					case AUDIO_STREAM_S ... AUDIO_STREAM_E:
						*apid = get_pid(buf+i+1);
						founda=1;
						break;
					}
				}
			}
			i += 187;
		}
		if (founda && foundb) break;
	}
}

void play_file_dvr_stdin(int fd_dvr, int fd, uint16_t apid, 
			 uint16_t vpid, int dev)
{
	char buf[BUFFY2];
	int count = 0, c;
	int written;
	uint16_t ppid=0;
	int fd_vdemux;
	int fd_ademux;
	int fda,fdv;
	uint16_t apid_n;
	uint16_t vpid_n;
	char devnamev[80];
	char devnamea[80];
	char devnamed[80];
	int set;

	sprintf(devnamev,"/dev/ost/video%d",dev);	
	sprintf(devnamea,"/dev/ost/audio%d",dev);
	sprintf(devnamed,"/dev/ost/demux%d",dev);

	if (nopat) find_avpids(STDIN_FILENO, &vpid, &apid);

	if (apid && vpid) ppid = 1;
	else {
		apid = 0;
		vpid = 0;
		ppid = 0;
	}

	if((fdv = open(devnamev,O_RDWR|O_NONBLOCK)) < 0){
		perror("VIDEO DEVICE: ");
		return;
	}
	    
	if((fda = open(devnamea,O_RDWR|O_NONBLOCK)) < 0){
		perror("AUDIO DEVICE: ");
		return;
	}
	

	fd_vdemux=open(devnamed, O_RDWR|O_NONBLOCK);
	fprintf(stderr,"vdemux open \n");
	fd_ademux=open(devnamed, O_RDWR|O_NONBLOCK);
	fprintf(stderr,"ademux open \n");

	set_pat_filt(fd);
	if (vpid && apid) set_av_filts(fd_vdemux,fd_ademux
				       ,vpid,apid);

	while ( (count = read(STDIN_FILENO,buf,BUFFY2)) > 0 ){
		written = 0;
		if (nopat) {
			set = 0;
			apid_n = 0;
			vpid_n = 0;
			find_bavpids(buf, count, &vpid_n, &apid_n);
			if (apid_n && apid_n != apid){
				apid = apid_n;
				set = 1;
			}
			if (vpid_n && vpid_n != vpid){
				vpid = vpid_n;
				set = 1;
			}
			if (set){
				fprintf(stderr,"reset pids %d %d\n",apid,vpid);
				set_av_filts(fd_vdemux,fd_ademux,vpid,apid);
			}
		}

		if(!ppid){
			ppid = get_pmt_pid(fd);
			if (ppid) set_pmt_filt(fd,ppid);
		} else if (!vpid && !apid){
			get_av_pids(fd,&vpid,&apid);
			fprintf(stderr,"apid: %d  vpid: %d\n",apid, vpid);
			videoStop(fdv);
			audioStop(fda);
			if (vpid && apid) set_av_filts(fd_vdemux,fd_ademux
						       ,vpid,apid);
			videoPlay(fdv);
			audioPlay(fda);
		}
		while( count > 0 && written < count){
			c = write(fd_dvr,buf+written, count-written);
			if (c>0) written += c;
		}
	}
}


void play_file_dvr(int filefd, int fd_dvr, int fd, uint16_t apid, 
		   uint16_t vpid, int dev)
{
	char buf[BUFFY2];
	int count, c;
	int written;
	int stopped = 0;
	struct pollfd pfd[NFD2];
	uint16_t ppid=0;
	int fd_vdemux;
	int fd_ademux;
	uint64_t length = 0;
	uint64_t l = 0;
	int fda,fdv;
	char devnamev[80];
	char devnamea[80];
	char devnamed[80];
	
	sprintf(devnamev,"/dev/ost/video%d",dev);	
	sprintf(devnamea,"/dev/ost/audio%d",dev);
	sprintf(devnamed,"/dev/ost/demux%d",dev);

	pfd[0].fd = STDIN_FILENO;
	pfd[0].events = POLLIN;
	
	if (nopat) find_avpids(filefd, &vpid, &apid);
	length = lseek(filefd, 0, SEEK_END);
	lseek(filefd,0,SEEK_SET);
	printf("File length: %2.2f MB\r\n",(double)length/(1024*1024));
	yskip++;

	if (apid && vpid) ppid = 1;
	else {
		apid = 0;
		vpid = 0;
		ppid = 0;
	}

	if((fdv = open(devnamev,O_RDWR|O_NONBLOCK)) < 0){
		perror("VIDEO DEVICE: ");
		return;
	}
	    
	if((fda = open(devnamea,O_RDWR|O_NONBLOCK)) < 0){
		perror("AUDIO DEVICE: ");
		return;
	}
	

	fd_vdemux=open(devnamed, O_RDWR|O_NONBLOCK);
	fd_ademux=open(devnamed, O_RDWR|O_NONBLOCK);
				
	set_pat_filt(fd);

	if (vpid && apid) set_av_filts(fd_vdemux, fd_ademux, vpid, apid);
	mvprintw(yskip+2,0,
		 "apid %d (0x%02x)",apid,apid);
	mvprintw(yskip+3,0,
		 "vpid %d (0x%02x)",vpid,vpid);

	
	set_volume(fda, volume);
	mvprintw(yskip+1,30,
		 "Volume: %d",volume);

	mvprintw(yskip,30, "playback started  ");
	refresh();

	while ( (count = read(filefd,buf,BUFFY2)) > 0  ){
		if (count > 0) l+= count;
		if (apid && vpid){
			mvprintw(yskip,0,"read %02.2f%%",100.0*l/length);
			refresh();
		}
		written = 0;
		if(!ppid && !nopat){
			ppid = get_pmt_pid(fd);
			if (ppid) set_pmt_filt(fd,ppid);
		} else if (!vpid && !apid){
			if (!nopat)
				get_av_pids(fd,&vpid,&apid);
			if (vpid && apid) 
				set_av_filts(fd_vdemux, fd_ademux, vpid, apid);
		}
		while( count > 0 && written < count){
			if (!stopped ){
				c = write(fd_dvr,buf+written, count-written);
				if (c>0) written += c;
			}
			if (poll(pfd,NFD2,100)){
				
				if (pfd[0].revents & POLLIN){
					int cc = getch();
					switch(cc){
					case 'z':
						mvprintw(yskip,30,
							 "playback frozen   ");
						videoFreeze(fdv);
						stopped = 1;
						break;
					case 'c':
						mvprintw(yskip,30,
							 "playback continued");
						videoContinue(fdv);
						stopped = 0;
						break;

					case '+':
						set_volume(fda, volume+1);
						mvprintw(yskip+1,30,
							 "Volume: %d",volume);
						break;

					case '-':
						set_volume(fda, volume-1);
						mvprintw(yskip+1,30,
							 "Volume: %d",volume);
						break;


					case 'q':
						videoContinue(fdv);
						exit(0);
						break;
					}
				}
				
			}
		}
	}
}


void write_out(uint8_t *buf, int count, void *priv)
{
	static int stopped = 0;
	struct pollfd pfd[2];
	ipack *p = (ipack *) priv;
	int written,c;
	int fd = p->fd;
	
	if (fd <0 ) return;

	pfd[1].fd = fd;
	pfd[1].events = POLLOUT;
	pfd[0].fd = STDIN_FILENO;
	pfd[0].events = POLLIN;

	
	set_volume(p->fd2, volume);
	mvprintw(yskip+1,30,"Volume: %d",volume);

	if(p->playing < 4){
		videoPlay(p->fd1);
		audioPlay(p->fd2);
		p->playing++;  
	}

	written = 0;
	while(written < count){
		if (poll(pfd,2,100)){
			if (pfd[1].revents & POLLOUT){
				c = write(fd,buf+written,count-written);
				if (c>0) written += c;
			}
   			if (pfd[0].revents & POLLIN){
				int c = getch();
				switch(c){
				case 'z':
					mvprintw(yskip,30,
						 "playback frozen    ");
					videoFreeze(p->fd1);
					stopped = 1;
					break;
				case 's':
					mvprintw(yskip,30,
						 "playback stopped   ");
					videoStop(p->fd1);
					stopped = 1;
					break;
					
				case 'c':
					mvprintw(yskip,30,
						 "playback continued ");
					audioSetAVSync(p->fd2, true);
					audioContinue(p->fd2);
					videoContinue(p->fd1);
					stopped = 0;
					break;
					
				case 'p':
					mvprintw(yskip,30,
						 "playback started   ");
					audioSetAVSync(p->fd2, true);
					audioPlay(p->fd2);
					videoPlay(p->fd1);
					stopped = 0;
					break;
					
				case 'f':
					mvprintw(yskip,30,
						 "fast forward       ");
					audioSetAVSync(p->fd2, false);
					audioPause(p->fd2);
					videoFastForward(p->fd1,0);
					stopped = 0;
					break;
					
				case 'm':
					mvprintw(yskip,30,
						 "slow motion        ");
					audioSetAVSync(p->fd2, false);
					audioPause(p->fd2);
					videoSlowMotion(p->fd1,2);
					stopped = 0;
					break;
					
				case '+':
					volume++;
					break;
					
				case '-':
					volume--;
					break;

				case 'q':
					videoContinue(p->fd1);
					p->playing=0;
					break;
				}
			}
			
		}
	}

}

void write_out_s(uint8_t *buf, int count, void *priv)
{
	struct pollfd pfd[1];
	int fd = (int) priv;
	int written, c;

	pfd[0].fd = fd;
	pfd[0].events = POLLOUT;

	written = 0;
	while(count > 0 && written < count){
		if (poll(pfd,1,100)){
			if (pfd[1].revents & POLLOUT){
				c = write(fd,buf+written,count-written);
				if (c>0) written += c;
			}
		}
	}
}


#define IPACKS 2048
void play_ts_video(int filefd, int fd, int fda, int next, uint16_t pida, 
		   uint16_t pidv)
{
	uint8_t buf[IN_SIZE];
	uint8_t mbuf[TS_SIZE];
	int count;
	uint64_t length = 0;
	uint64_t l = 0;
	int i;
	uint16_t pid;
	ipack pa, pv;
	ipack *p;



	videoSetBlank(fd,false);
	if (!next){
		audioSelectSource(fda,AUDIO_SOURCE_MEMORY);
		videoSelectSource(fd,VIDEO_SOURCE_MEMORY);
	}
	audioSetAVSync(fda, true);

	if (nopat) 
		find_avpids(filefd, &pidv, &pida);


	if (filefd == STDIN_FILENO){
		init_ipack(&pa, IPACKS, write_out_s);
		init_ipack(&pv, IPACKS, write_out_s);
		length = 1;
	} else {
		init_ipack(&pa, IPACKS, write_out);
		init_ipack(&pv, IPACKS, write_out);
		length = lseek(filefd, 0, SEEK_END);
		lseek(filefd,0,SEEK_SET);
		printf("File length: %2.2f MB\r\n",
			(double)length/(1024*1024));
		yskip++;
		refresh();

	}
	pa.data = (void *)&pa;
	pv.data = (void *)&pv;
	pv.fd   = fd;
	pa.fd   = fda;
	pv.fd1  = fd;
	pa.fd1  = fd;
	pv.fd2  = fda;
	pa.fd2  = fda;

	count = read(filefd,mbuf,TS_SIZE);
	l+= count;
	for ( i = 0; i < 188 ; i++){
		if ( mbuf[i] == 0x47 ) break;
	}

	if ( i == 188){
		printf("Not a TS\n");
		yskip++;
		refresh();
	} else {
		memcpy(buf,mbuf+i,TS_SIZE-i);
		count = read(filefd,mbuf,i);
		l+= count;
		memcpy(buf+TS_SIZE-i,mbuf,i);
		i = 188;
	}
	
	pv.playing = 1;
	pa.playing = 1;
	mvprintw(yskip,30, "playback started  ");
	while ( (l < length || filefd == STDIN_FILENO)
		&& pv.playing && pa.playing){
		if ( ( count = read(filefd,buf+i,IN_SIZE-i)) < 0){
		        perror("ts play ");
			exit(1);
		}		
		if (count < 1) break;

		l+= count;
		mvprintw(yskip,0,"read %02.2f%%",100.0*l/length);
		refresh();

		for( i = 0; i < count; i+= TS_SIZE){
			uint8_t off = 0;

			if ( count - i < TS_SIZE) break;

			pid = get_pid(buf+i+1);
			if (!(buf[3+i]&0x10)) // no payload?
				continue;
			if (pid == pidv){
				p = &pv;
			} else {
				if (pid == pida){
					p = &pa;
				} else continue;
			}
			if ( buf[1+i]&0x40) {
				if (p->plength == MMAX_PLENGTH-6){
					p->plength = p->found-6;
					p->found = 0;
					send_ipack(p);
					reset_ipack(p);
				}
			}

			if ( buf[3+i] & 0x20) {  // adaptation field?
				off = buf[4+i] + 1;
			}
			instant_repack(buf+4+off+i, TS_SIZE-4-off, p);
		}
		i = 0;

	}

	audioSelectSource(fda,AUDIO_SOURCE_DEMUX);
	videoSelectSource(fd,VIDEO_SOURCE_DEMUX);
	videoPlay(fd);
	audioSetAVSync(fda, true);
}


void usage(char *progname)
{
	fprintf(stderr,"usage: %s [options] <input files>\n\n",progname);
	fprintf(stderr,"options:\n");
	fprintf(stderr,"  --help,             -h:  print help message\n");  
	fprintf(stderr,"  --input_stream,     -i:  set input stream type (PS_STREAM (default), TS_STREAM)\n");
	fprintf(stderr,"  --audio_out,        -e:  set audio stream type (MP2(default), AC3 and others)\n");
	fprintf(stderr,"  --audio_pid,        -a:  audio PID for TS stream (0 = auto)\n");
	fprintf(stderr,"  --video_pid,        -v:  audio PID for TS stream (0 = auto)\n");
	fprintf(stderr,"  --device,           -d:  device nb.(0 = default)\n");
	fprintf(stderr,"  --no_dvr,           -n:  do not use dvr device\n");
	fprintf(stderr,"  --stdin,            -o:  read from stdin\n");
	fprintf(stderr,"  --nopat,            -s:  try to find apid and vpid without PAT\n");
	exit(1);
}



int main(int argc, char **argv)
{
	int fda,fdv;
	int filefd=0;
	int i, c;
	uint16_t vpid = 0;
	uint16_t apid = 0;
	char *inpt = "PS_STREAM";
	char *audiot = "MP2";
	int dev =0;
	char devnamed[80];
	char devnamedv[80];
	int dvr = 1;
	int fd,fd_dvr;
	int stin = 0;

	while (1) {
		int option_index = 0;
		static struct option long_options[] =
		{
		 {"input_stream", required_argument, NULL, 'i'},
		 {"audio_out", required_argument, NULL, 'e'},
		 {"video_pid", required_argument, NULL, 'v'},
		 {"audio_pid", required_argument, NULL, 'a'},
		 {"device", required_argument, NULL, 'd'},
		 {"no_dvr", no_argument, NULL, 'n'},
		 {"stdin", no_argument, NULL, 'o'},
		 {"nopat", no_argument, NULL, 's'},
		 {"help", no_argument , NULL, 'h'},
		 {0, 0, 0, 0}
		};

		c = getopt_long (argc, argv, "i:hv:a:d:noe:s",
				 long_options, &option_index);
		if (c == -1)
			break;
		
		switch (c) {
		case 'i':
			inpt = optarg;
			break;
		case 'e':
			audiot = optarg;
			break;
		case 'v':
			vpid = atoi(optarg);
			inpt = "TS_STREAM";
			break;

		case 'a':
			apid = atoi(optarg);
			inpt = "TS_STREAM";
			break;

		case 'd':
			dev = atoi(optarg);
			break;
		case 'n':
			dvr = 0;
			inpt = "TS_STREAM";
			break;
		case 'o':
			stin = 1;
			break;
		case 's':
			nopat = 1;
			inpt = "TS_STREAM";
			break;

		case 'h':
		case '?':
		default:
			usage(argv[0]);
		}
	}
	
	sprintf(devnamed,"/dev/ost/demux%d",dev);	
	sprintf(devnamedv,"/dev/ost/dvr%d",dev);	
	
	
	yskip = 12;
	if ( !strcmp(inpt, "PS_STREAM")){
		if (open_av(&fdv, &fda, dev) < 0) exit(1);	
		if (audioSetStreamtype(fda, audiot) < 0) exit(1);	
		if (videoSetStreamtype(fdv, "PROG") < 0) exit(1);	

		if (stin) { 
			play_file_video_stdin(fdv, fda);
		} else {
			curses_init();
			printf("Welcome to NTuxplayer\r\n\n");
			printf("c: continue playback\r\n");
			printf("f: fast forward\r\n");
			printf("m: slow motion\r\n");
			printf("p: playback\r\n");
			printf("q: quit\r\n");
			printf("s: stop playback\r\n");
			printf("z: freeze picture\r\n");
			printf("+: increase volume\r\n");
			printf("-: decrease volume\r\n");
			refresh();

			for ( i=optind; i< argc; i++){
				if ( (filefd = open(argv[i],O_RDONLY)) < 0){
					perror("File open:");
					return -1;
				}
				printf("Playing %s\r\n",argv[i]);
				yskip++;
				refresh();
				play_file_video(filefd, fdv, fda, i-optind );
			
				close(filefd);
			}
		}
		if (fda) audioSetAVSync(fda, true);

		close(fdv);
		close(fda);
	} else if ( !strcmp(inpt, "TS_STREAM") ){

		switch (dvr){
		case 1:
			if((fd = open(devnamed,O_RDWR|O_NONBLOCK)) < 0){
				perror("DEMUX DEVICE: ");
				return -1;
			}
			
			if((fd_dvr = open(devnamedv,O_WRONLY)) < 0){
				perror("DVR DEVICE: ");
				return -1;
			}
			
			if (stin) { 
				play_file_dvr_stdin(fd_dvr, fd, apid, vpid, 
						    dev);
			} else {
				curses_init();
				yskip -= 4;
				printf("Welcome to NTuxplayer\r\n\n");
				printf("c: continue playback\r\n");
				printf("q: quit\r\n");
				printf("z: freeze picture\r\n");
				printf("+: increase volume\r\n");
				printf("-: increase volume\r\n");
				refresh();
				for ( i=optind; i< argc; i++){
					if ( (filefd = 
					      open(argv[i],O_RDONLY)) < 0){
						perror("File open:");
						return -1;
					}
					printf("Playing %s\r\n",argv[i]);
					yskip++;
					refresh();
					play_file_dvr(filefd, fd_dvr, fd, 
						      apid, vpid, dev);
					
					close(filefd);
				}
			}
			close(fd);
			close(fd_dvr);
			break;
		case 0:
			if (open_av(&fdv, &fda, dev) < 0) exit(1);	
			if (audioSetStreamtype(fda, audiot) < 0) exit(1);
			if (videoSetStreamtype(fdv, "MPEG2") < 0) exit(1);
			if (stin) { 
				play_ts_video(STDIN_FILENO, fdv, fda, 0, 
					      apid, vpid);
			} else {
				curses_init();
				printf("Welcome to NTuxplayer\r\n\n");
				printf("c: continue playback\r\n");
				printf("f: fast forward\r\n");
				printf("m: slow motion\r\n");
				printf("p: playback\r\n");
				printf("q: quit\r\n");
				printf("s: stop playback\r\n");
				printf("z: freeze picture\r\n");
				printf("+: increase volume\r\n");
				printf("-: increase volume\r\n");
				refresh();

				for ( i=optind; i< argc; i++){
					if ( (filefd = 
					      open(argv[i],O_RDONLY)) < 0){
						perror("File open:");
						return -1;
					}
					printf("Playing %s\r\n",argv[i]);
					yskip++;
					refresh();
					play_ts_video(filefd, fdv, fda, 
						      i-optind, apid, vpid);
					
					close(filefd);
				}
			}
			if (fda) audioSetAVSync(fda, true);

			close(fdv);
			close(fda);
			break;
		}
	}

	return 0;


}

