#ifndef __DVBTYPES_H__
#define __DVBTYPES_H__

#include <asm/types.h>

typedef enum BOOTSTATES
{
	BOOTSTATE_BUFFER_EMPTY	= 0,
	BOOTSTATE_BUFFER_FULL	= 1,
	BOOTSTATE_BOOT_COMPLETE	= 2
} BOOTSTATES;

typedef enum GPIO_MODE
{
	GPIO_INPUT = 0x00,
	GPIO_IRQHI = 0x10,
	GPIO_IRQLO = 0x20,
	GPIO_IRQHL = 0x30,
	GPIO_OUTLO = 0x40,
	GPIO_OUTHI = 0x50
} GPIO_MODE;

typedef enum
{	RP_None,
	AudioPES,
	AudioMp2,
	AudioPCM,
	VideoPES,
	AV_PES
} TYPE_REC_PLAY_FORMAT;

typedef struct PARAMSTRUCT
{
	unsigned int	wCommand;
        int             error;
	unsigned long	pdwData[100];
} PARAMSTRUCT, *PPARAMSTRUCT;

typedef enum OSDPALTYPE
{
	NoPalet =  0,      /* No palette */
	Pal1Bit =  2,      /* 2 colors for 1 Bit Palette    */
	Pal2Bit =  4,      /* 4 colors for 2 bit palette    */
	Pal4Bit =  16,     /* 16 colors for 4 bit palette   */
	Pal8Bit =  256     /* 256 colors for 16 bit palette */
} OSDPALTYPE, *POSDPALTYPE;

typedef enum {
   BITMAP1,           /* 1 bit bitmap */
   BITMAP2,           /* 2 bit bitmap */
   BITMAP4,           /* 4 bit bitmap */
   BITMAP8,           /* 8 bit bitmap */
   BITMAP1HR,         /* 1 Bit bitmap half resolution */
   BITMAP2HR,         /* 2 bit bitmap half resolution */
   BITMAP4HR,         /* 4 bit bitmap half resolution */
   BITMAP8HR,         /* 8 bit bitmap half resolution */
   YCRCB422,          /* 4:2:2 YCRCB Graphic Display */
   YCRCB444,          /* 4:4:4 YCRCB Graphic Display */
   YCRCB444HR,        /* 4:4:4 YCRCB graphic half resolution */
   VIDEOTSIZE,        /* True Size Normal MPEG Video Display */
   VIDEOHSIZE,        /* MPEG Video Display Half Resolution */
   VIDEOQSIZE,        /* MPEG Video Display Quarter Resolution */
   VIDEODSIZE,        /* MPEG Video Display Double Resolution */
   VIDEOTHSIZE,       /* True Size MPEG Video Display Half Resolution */
   VIDEOTQSIZE,       /* True Size MPEG Video Display Quarter Resolution*/
   VIDEOTDSIZE,       /* True Size MPEG Video Display Double Resolution */
   VIDEONSIZE,        /* Full Size MPEG Video Display */
   CURSOR             /* Cursor */
} DISPTYPE;           /* Window display type           */

// switch defines
#define SB_GPIO 3
#define SB_OFF	GPIO_OUTLO  //SlowBlank aus (TV-Mode)
#define SB_ON	GPIO_INPUT  //SlowBlank an  (AV-Mode)
#define SB_WIDE GPIO_OUTHI  //SlowBlank 6V  (16/9-Mode) nicht realisiert

#define FB_GPIO 1
#define FB_OFF	GPIO_LO     //FastBlank aus (CVBS-Mode)
#define FB_ON   GPIO_OUTHI  //FastBlank an  (RGB-Mode)
#define FB_LOOP	GPIO_INPUT  //FastBlank der PC-Grafik durchschleifen

typedef enum VIDEOOUTPUTMODE
{
        NO_OUT       = 0,		//disable analog Output
	CVBS_RGB_OUT = 1,
	CVBS_YC_OUT  = 2,
	YC_OUT	     = 3
} VIDEOOUTPUTMODE, *PVIDEOOUTPUTMODE;

typedef enum VIDEOTESTMODE
{
	TEST_BAR_OFF = 0,
	TEST_BAR_1   = 1,
	TEST_BAR_2   = 2
} VIDEOTESTMODE, *PVIDEOTESTMODE;

typedef struct VIDEOSTATE
{
	__u16	ProcessingState;		//0
	__u16	CommandID;
	__u16	dummy1;					//2
	__u16	dummy2;
	__u16	RateBuffFullness;		//4
	__u16	dummy3;
	__u16	BytesDecoded;			//6
	__u16	SkippedPictures;
	__u16	RepeatedPictures;		//8
	__u16	MostRecentPTS;
	__u16	LastPicture;			//10
	__u16	InitDone;
	__u16	FreezeIndex;			//12
	__u16	FindIndex;
	__u16	DistanceI;				//14
	__u16	ThresholdPTS;
	__u16	dummy4;					//16
	__u16	dummy5;
	__u16	DisablePTSFilt;			//18
	__u16	HSize;
	__u16	VSize;					//20
	__u16	AspectRatio;
	__u16	FrameRate;				//22
	__u16	BitRate;
	__u16	VBVBuffSize;			//24
	__u16	ConstParamFlag;
	__u16	IntraQ;					//26
	__u16	NonIntraQ;
	__u16	FrameInterval;			//28
	__u16	HeaderBackup;
	__u16	RedHeaderFlag;			//30
	__u16	SeqExtension;
	__u16	GOPTimeCode1;			//32
	__u16	GOPTimeCode2;
	__u16	ClosedGOP;				//34
	__u16	PICHeadTempRef;
	__u16	SegHeaderExt;			//36
	__u16	ColorDesc;
	__u16	ColorPrim;				//38
	__u16	TransferChar;
	__u16	MatrixCoeff;			//40
	__u16	DisplayHSize;
	__u16	DisplayVSize;			//42
	__u16	GOPHeader;
	__u16	TimeCodeW1;				//44
	__u16	TimeCodeW2;
} VIDEOSTATE, *PVIDEOSTATE;

#endif







































